--[[
Copyright (c) 2000-2019 Synology Inc. All rights reserved.
]]--

--[[
Rspamd filter priority:
prefilter: MCP(10), extract MCP(9), AntiVirus whitelist(6), AntiVirus(5), Blacklist/Whitelist(2), cache(1)
filter: spam, SpamAssassin rules
postfilter: header tagging + cache(10)
]] --

local rspamd_logger = require "rspamd_logger"

local local_conf = rspamd_paths['CONFDIR']
local cache = dofile(local_conf .. '/cache.lua')

local SYNOLOGY_SPAM_ACTION = 'synology_spam'
local SYNOLOGY_ANTIVIRUS_ACTION = 'synology_antivirus'
local SYNOLOGY_MCP_ACTION = 'synology_mcp'

local SPAM_STATUS_HEADER = 'X-Synology-Spam-Status'
local SPAM_FLAG_HEADER = 'X-Synology-Spam-Flag'
local VIRUS_HEADER = 'X-Synology-Virus-Status'
local MCP_HEADER = 'X-Synology-MCP-Status'

local MCP_SYMBOL_PREFIX = 'SYNO_MCP_'
local MCP_SCORE_SYMBOL = 'SYNO_MCP_SCORE'
local ANTIVIRUS_SYMBOLS = {'CLAM_VIRUS', 'SYNOAV_MCAFEE_VIRUS'}
local ANTIVIRUS_SYMBOL_SUFFIXES = {
  '',
  -- '_FAIL', -- failure symbol doesn't means email is virus
  '_ENCRYPTED', -- Consider encrypted symbol as virus for now
}
local ANTIVIRUS_WHITELIST_SYMBOLS = {
  'ANTIVIRUS_WHITELIST_SENDER_IP',
  'ANTIVIRUS_WHITELIST_SENDER_DOMAIN',
  'ANTIVIRUS_WHITELIST_SENDER_EMAIL',
  'ANTIVIRUS_WHITELIST_RECIPIENT_EMAIL'
}
local DKIM_SPAM_SYMBOL = 'SYNO_DKIM_HEADER_SPAM'
local DMARC_QUARANTINE_SYMBOL = 'SYNO_DMARC_HEADER_QUARANTINE'

local SPAM_BLACKLIST_PREFIX = 'BLACKLIST'
local SPAM_WHITELIST_PREFIX = 'WHITELIST'
local SPAM_BLACKWHITE_INFIXES = {
  'FROM',
  'TO'
}
local SPAM_BLACKWHITE_SUFFIXES = {
  'ACCOUNT',
  'ADDRESS',
  'DOMAIN'
}

local function is_prefix(str, prefix)
  return string.sub(str, 1, string.len(prefix)) == prefix
end

local function is_mcp_symbol(symbol)
  return is_prefix(symbol['name'], MCP_SYMBOL_PREFIX)
end

local function is_antivirus_symbol(symbol)
  for i, antivirus_symbol in ipairs(ANTIVIRUS_SYMBOLS) do
    if is_prefix(symbol['name'], antivirus_symbol) then
      for j, antivirus_symbol_suffix in ipairs(ANTIVIRUS_SYMBOL_SUFFIXES) do
        -- Directly checking full antivirus symbol instead of checking suffix
        -- Because '' suffix will make suffix checking difficult
        if symbol['name'] == antivirus_symbol .. antivirus_symbol_suffix then
          return true
        end
      end
    end
  end

  return false
end

local function is_antivirus_whitelist_symbol(symbol)
  for i, antivirus_whitelist_symbol in ipairs(ANTIVIRUS_WHITELIST_SYMBOLS) do
    if antivirus_whitelist_symbol == symbol['name'] then
      return true
    end
  end

  return false
end

local function is_spam_symbol(symbol)
  return not is_mcp_symbol(symbol) and
  not is_antivirus_symbol(symbol) and
  not is_antivirus_whitelist_symbol(symbol)
end

local function is_whitelist_symbol(symbol)
  local symbol_parts = rspamd_str_split(symbol['name'], '_')
  if #symbol_parts ~= 3 then
    return false
  end
  if symbol_parts[1] ~= SPAM_WHITELIST_PREFIX then
    return false
  end
  local infix_match = false
  for i, spam_blackwhite_infix in ipairs(SPAM_BLACKWHITE_INFIXES) do
    if spam_blackwhite_infix == symbol_parts[2] then
      infix_match = true
      break
    end
  end
  local suffix_match = false
  for i, spam_blackwhite_suffix in ipairs(SPAM_BLACKWHITE_SUFFIXES) do
    if spam_blackwhite_suffix == symbol_parts[3] then
      suffix_match = true
      break
    end
  end
  return infix_match and suffix_match
end

local function is_blacklist_symbol(symbol)
  local symbol_parts = rspamd_str_split(symbol['name'], '_')
  if #symbol_parts ~= 3 then
    return false
  end
  if symbol_parts[1] ~= SPAM_BLACKLIST_PREFIX then
    return false
  end
  local infix_match = false
  for i, spam_blackwhite_infix in ipairs(SPAM_BLACKWHITE_INFIXES) do
    if spam_blackwhite_infix == symbol_parts[2] then
      infix_match = true
      break
    end
  end
  local suffix_match = false
  for i, spam_blackwhite_suffix in ipairs(SPAM_BLACKWHITE_SUFFIXES) do
    if spam_blackwhite_suffix == symbol_parts[3] then
      suffix_match = true
      break
    end
  end
  return infix_match and suffix_match
end

local function is_dkim_spam_symbol(symbol)
  return symbol['name'] == DKIM_SPAM_SYMBOL
end

local function is_dmarc_quarantine_symbol(symbol)
  return symbol['name'] == DMARC_QUARANTINE_SYMBOL
end

local function spam_check(task, add, del)
  local spam_opt = rspamd_config:get_all_opt('spam')
  local spam_enabled = spam_opt['enabled']
  local spam_learn_enabled = spam_opt['learn_enabled']

  if not spam_enabled then
    return
  end

  local action = task:get_metric_action('default')
  local score = task:get_metric_score('default')

  local symbols = task:get_symbols_all()
  -- symbol: {[group] = group, [options] = {[1] = option1, [2] = option2}, [name] = name, [score] = score}
  -- whitelist
  for i, symbol in ipairs(symbols) do
    if is_whitelist_symbol(symbol) then
      add[SPAM_FLAG_HEADER] = 'no'
      add[SPAM_STATUS_HEADER] = 'score=0, required ' .. score[2] .. ', ' .. symbol['name'] .. ' 0'
      rspamd_logger.infox(task, "Hit whitelist; symbol %s", symbol)
      return
    end
  end
  -- blacklist
  for i, symbol in ipairs(symbols) do
    if is_blacklist_symbol(symbol) then
      add[SPAM_FLAG_HEADER] = 'yes'
      add[SPAM_STATUS_HEADER] = 'score=' .. score[2] .. ', required ' .. score[2] .. ', ' .. symbol['name'] .. ' ' .. score[2]
      rspamd_logger.infox(task, "Hit blacklist; symbol %s", symbol)
      return
    end
  end
  -- DKIM fail
  for i, symbol in ipairs(symbols) do
    if is_dkim_spam_symbol(symbol) then
      add[SPAM_FLAG_HEADER] = 'yes'
      add[SPAM_STATUS_HEADER] = 'score=' .. score[2] .. ', required ' .. score[2] .. ', ' .. symbol['name'] .. ' ' .. score[2]
      rspamd_logger.infox(task, "Spammed by DKIM; symbol %s", symbol)
      return
    end
  end
  -- DMARC quarantine
  for i, symbol in ipairs(symbols) do
    if is_dmarc_quarantine_symbol(symbol) then
      add[SPAM_FLAG_HEADER] = 'yes'
      add[SPAM_STATUS_HEADER] = 'score=' .. score[2] .. ', required ' .. score[2] .. ', ' .. symbol['name'] .. ' ' .. score[2]
      rspamd_logger.infox(task, "Quarantined by DMARC; symbol %s", symbol)
      return
    end
  end

  -- score = {[1] = current score, [2] = required score }
  local spam_status
  local cache_hit = false
  if cache.enabled then
    local cache_symbol = task:get_symbol(cache.settings.symbol)
    if cache_symbol ~= nil then
      rspamd_logger.infox(task, "Cache hit; getting spam status from cache")
      cache_hit = true

      -- split cache_symbol into flag and status with semicolon
      local cache_spam_result = cache_symbol[1]['options'][1]
      local sep_index = string.find(cache_spam_result, ';')
      if sep_index then
        add[SPAM_FLAG_HEADER] = string.sub(cache_spam_result, 1, sep_index - 1)
        spam_status = string.sub(cache_spam_result, sep_index + 1)
      end
    end
  end

  if spam_status == nil then
    if cache.enabled then
      rspamd_logger.infox(task, "Cache miss; calculate spam status")
    end
    spam_status = 'score=' .. score[1] .. ', required ' .. score[2]
    if spam_learn_enabled then
      local spam_learn_threshold_spam = spam_opt['learn_threshold_spam']
      local spam_learn_threshold_non_spam = spam_opt['learn_threshold_non_spam']

      if score[1] >= spam_learn_threshold_spam then
        spam_status = spam_status .. ', autolearn=spam'
      elseif score[1] <= spam_learn_threshold_non_spam then
        spam_status = spam_status .. ', autolearn=ham'
      end
    end

    for i, symbol in ipairs(symbols) do
      if is_spam_symbol(symbol) then
        spam_status = spam_status .. ', ' .. symbol['name'] .. ' ' .. symbol['score']
      end
    end

    if action == SYNOLOGY_SPAM_ACTION then
      add[SPAM_FLAG_HEADER] = 'yes'
    else
      add[SPAM_FLAG_HEADER] = 'no'
    end
  end

  add[SPAM_STATUS_HEADER] = spam_status

  rspamd_logger.infox(task, "Spam Flag: %1", add[SPAM_FLAG_HEADER])
  rspamd_logger.infox(task, "Spam Status: %1", add[SPAM_STATUS_HEADER])
  if add[SPAM_FLAG_HEADER] == 'yes' then
    rspamd_logger.errx(task, "msgid=<%1>: Spam, %2", task:get_message_id(), add[SPAM_STATUS_HEADER])
  end

  -- add spam result to cache if not in cache
  if cache.enabled and not cache_hit then
    -- join flag and status into cache_symbol with semicolon
    cache.set(task, add[SPAM_FLAG_HEADER] .. ';' .. add[SPAM_STATUS_HEADER])
  end
end

local function antivirus_check(task, add, del)
  local antivirus_opt = rspamd_config:get_all_opt('antivirus')
  local antivirus_enabled = antivirus_opt['enabled']

  if not antivirus_enabled then
    return
  end

  local action = task:get_metric_action('default')
  if action ~= SYNOLOGY_ANTIVIRUS_ACTION then
    add[VIRUS_HEADER] = 'no'
    return
  end

  local virus_names

  for i, virus_symbol in ipairs(ANTIVIRUS_SYMBOLS) do
    for j, antivirus_symbol_suffix in ipairs(ANTIVIRUS_SYMBOL_SUFFIXES) do
      local virus = task:get_symbol(virus_symbol .. antivirus_symbol_suffix)
      if virus then
        virus_names = virus[1]['options']
        break
      end
    end
  end

  if action == SYNOLOGY_ANTIVIRUS_ACTION then
    virus_names = table.concat(virus_names, ', ')
    add[VIRUS_HEADER] = 'yes, ' .. virus_names
    rspamd_logger.errx(task, "msgid=<%1>, Virus, %2", task:get_message_id(), virus_names)
  end
end

local function mcp_check(task, add, del)
  local mcp_opt = rspamd_config:get_all_opt('mcp')
  local mcp_enabled = mcp_opt['enabled']

  if not mcp_enabled then
    return
  end

  local mcp_score_required = mcp_opt['actions'][SYNOLOGY_MCP_ACTION]
  local mcp_score = 0
  local mcp_symbols = {}
  local symbols = task:get_symbols_all()

  for i, symbol in ipairs(symbols) do
    if is_mcp_symbol(symbol) then
      if symbol['name'] == MCP_SCORE_SYMBOL then
        mcp_score = tonumber(symbol['options'][1])
      else
        mcp_symbols[#mcp_symbols + 1] = symbol['name']
      end
    end
  end

  if mcp_score >= mcp_score_required then
    local mcp_header = 'yes, score=' .. mcp_score .. ', required ' .. mcp_score_required
    local mcp_symbol_string = table.concat(mcp_symbols, ', ')

    add[MCP_HEADER] = mcp_header .. ', ' .. mcp_symbol_string
    rspamd_logger.errx(task, "msgid=<%1>, MCP, %2", task:get_message_id(), mcp_symbol_string)
  else
    add[MCP_HEADER] = 'no'
  end
end

-- Multiple Headers
rspamd_config:register_symbol({
    name = 'SYNOLOGY_MCP_HEADER',
    type = 'prefilter',
    priority = 9,
    callback = function(task)
      -- Set total MCP score to MCP_SCORE_SYMBOL's option
      -- Remove total MCP score from default metric
      local mcp_opt = rspamd_config:get_all_opt('mcp')
      local mcp_enabled = mcp_opt['enabled']

      if not mcp_enabled then
        return
      end

      local mcp_score = 0
      local mcp_symbols = {}
      local symbols = task:get_symbols_all()

      for i, symbol in ipairs(symbols) do
        if is_mcp_symbol(symbol) then
          mcp_score = mcp_score + symbol['score']
        end
      end

      task:insert_result(MCP_SCORE_SYMBOL, 0.0, tostring(mcp_score))

      -- remove mcp score
      local score = task:get_metric_score('default') -- score = {[1] = current score, [2] = required score }
      task:set_metric_score('default', score[1] - mcp_score)
    end
  })

-- Multiple Headers
rspamd_config:register_symbol({
    name = 'SYNOLOGY_HEADERS',
    type = 'postfilter',
    priority = 10,
    callback = function(task)
      local add = {}
      local del = {
        ['X-Spam'] = 1,
        ['X-Virus'] = 1,
      }

      -- mcp
      mcp_check(task, add, del)

      -- spam
      spam_check(task, add, del)

      -- virus
      antivirus_check(task, add, del)

      task:set_milter_reply({
          add_headers = add,
          remove_headers = del,
        })
    end
  })
